`default_nettype none

module DE1_SoC(

      ///////// aud /////////
      input  wire        aud_adcdat,
      output wire        aud_adclrck,
      output wire        aud_bclk,
      output logic       aud_dacdat,
      output wire        aud_daclrck,
      output logic       aud_mclk,

      ///////// clock /////////
      input  wire        clock_50,

      ///////// fpga /////////
      output logic       fpga_i2c_sclk,
      inout  wire        fpga_i2c_sdat,

      ///////// gpio /////////
      inout  wire [35:0] gpio_0,
      inout  wire [35:0] gpio_1,


      ///////// hex0 /////////
      output logic[6:0]  hex0,

      ///////// hex1 /////////
      output logic[6:0]  hex1,

      ///////// hex2 /////////
      output logic[6:0]  hex2,

      ///////// hex3 /////////
      output logic[6:0]  hex3,

      ///////// hex4 /////////
      output logic[6:0]  hex4,

      ///////// hex5 /////////
      output logic[6:0]  hex5,

      ///////// key /////////
      input  wire [3:0]  key,

      ///////// ledr /////////
      output logic[9:0]  ledr,

      ///////// sw /////////
      input  wire [9:0]  sw
);


   // reset generation
   logic             reset_n;
   gene_reset gene_reset(.clk(clock_50), .reset_n(reset_n));

   // 1s slow clock (led blink)
   logic             clk_1s;
   gene_clk gene_clkl(.clk_50(clock_50), .clk_1s(clk_1s));



   // Turn on all displays except LCD
   assign  hex0            =       0;
   assign  hex1            =       0;
   assign  hex2            =       0;
   assign  hex3            =       0;
   assign  hex4            =       0;
   assign  hex5            =       0;
	assign  ledr            =       (sw | key) ^ clk_1s;


   assign  gpio_0          =       36'hzzzzzzzzz;
   assign  gpio_1          =       36'hzzzzzzzzz;

   /***************************************************************************/
   // Communication with the WM8731 Audio Codec
   // DO NOT CHANGE !!
   /***************************************************************************/
   logic signed [15:0]      dac_data_r, dac_data_l, adc_data_r, adc_data_l;
   logic                    audio_data_ena;
   codec codec(.clk_50(clock_50),
               .reset_n(reset_n),

               // I2C for configuration
               .i2c_sclk(fpga_i2c_sclk),
               .i2c_sdat(fpga_i2c_sdat),

               // CODEC data ports
               .aud_adclrck(aud_adclrck),
               .aud_adcdat(aud_adcdat),
               .aud_daclrck(aud_daclrck),
               .aud_dacdat(aud_dacdat),
               .aud_bclk(aud_bclk),
               .aud_mclk(aud_mclk),

               // Audio data :
               // - audio clock "aud_mclk" is set to 12MHz,
               // - sampling rate is set to 48Khz (250 cycles between 2 audio samples)
               // - signal suffixed with "l" correspond to the left channel
               // - signal suffixed with "r" correspond to the right channel
               // - audio_data_enable indicate that a new pair of samples from
               // the ADC are available (will go high during one clock cycle)
               // - at the same moment, this module will capture a pair of
               // sample to send to the DAC
               .adc_data_l(adc_data_l),
               .adc_data_r(adc_data_r),
               .dac_data_l(dac_data_l),
               .dac_data_r(dac_data_r),
               .data_ena(audio_data_ena)
               );

   /***************************************************************************
             STUDENT MODULE
    **************************************************************************/

   // audio processing
   audio_proc audio_proc_i(
                  .clk(aud_mclk),
                  .reset_n(reset_n),
                  .audio_data_ena(audio_data_ena),
                  .adc_data_l(adc_data_l),
                  .adc_data_r(adc_data_r),
                  .dac_data_l(dac_data_l),
                  .dac_data_r(dac_data_r)
                  );


endmodule
